@@ -1,5 +1,6 @@
 import logging
 from dataclasses import asdict
+from datetime import timedelta
 from typing import Any
 from typing import Dict
 from typing import List
@@ -10,6 +11,9 @@ from typing import Union
 import httpx
 
 from autoblocks._impl.api.models import AbsoluteTimeFilter
+from autoblocks._impl.api.models import Dataset
+from autoblocks._impl.api.models import DatasetItem
+from autoblocks._impl.api.models import DatasetWithItems
 from autoblocks._impl.api.models import Event
 from autoblocks._impl.api.models import RelativeTimeFilter
 from autoblocks._impl.api.models import Trace
@@ -17,29 +21,31 @@ from autoblocks._impl.api.models import TraceFilter
 from autoblocks._impl.api.models import TracesResponse
 from autoblocks._impl.api.models import View
 from autoblocks._impl.config.constants import API_ENDPOINT
+from autoblocks._impl.util import AutoblocksEnvVar
 
 log = logging.getLogger(__name__)
 
 
+def make_trace_response(data: Dict) -> Trace:
+    return Trace(
+        id=data["id"],
+        events=[
+            Event(
+                id=event["id"],
+                trace_id=event["traceId"],
+                message=event["message"],
+                timestamp=event["timestamp"],
+                properties=event.get("properties") or {},
+            )
+            for event in data["events"]
+        ],
+    )
+
+
 def make_traces_response(data: Dict) -> TracesResponse:
     return TracesResponse(
         next_cursor=data.get("nextCursor"),
-        traces=[
-            Trace(
-                id=trace["id"],
-                events=[
-                    Event(
-                        id=event["id"],
-                        trace_id=trace["id"],
-                        message=event["message"],
-                        timestamp=event["timestamp"],
-                        properties=event.get("properties") or {},
-                    )
-                    for event in trace["events"]
-                ],
-            )
-            for trace in data["traces"]
-        ],
+        traces=[make_trace_response(trace) for trace in data["traces"]],
     )
 
 
@@ -52,10 +58,14 @@ def camel_case_factory(values: List[Tuple[str, Any]]) -> Dict:
 
 
 class AutoblocksAPIClient:
-    def __init__(self, api_key: str) -> None:
+    def __init__(self, api_key: Optional[str] = None, timeout: timedelta = timedelta(seconds=10)) -> None:
+        api_key = api_key or AutoblocksEnvVar.API_KEY.get()
+        if not api_key:
+            raise ValueError(f"You must provide an api_key or set the {AutoblocksEnvVar.API_KEY} environment variable.")
         self._client = httpx.Client(
             base_url=API_ENDPOINT,
             headers={"Authorization": f"Bearer {api_key}"},
+            timeout=timeout.total_seconds(),
         )
 
     def get_views(self) -> List[View]:
@@ -64,6 +74,11 @@ class AutoblocksAPIClient:
         resp = req.json()
         return [View(id=view["id"], name=view["name"]) for view in resp]
 
+    def get_trace(self, trace_id: str) -> Trace:
+        req = self._client.get(f"/traces/{trace_id}")
+        req.raise_for_status()
+        return make_trace_response(req.json())
+
     def get_traces_from_view(self, view_id: str, *, page_size: int, cursor: Optional[str] = None) -> TracesResponse:
         req = self._client.get(
             f"/views/{view_id}/traces",
@@ -95,3 +110,19 @@ class AutoblocksAPIClient:
         )
         req.raise_for_status()
         return make_traces_response(req.json())
+
+    def get_datasets(self) -> List[Dataset]:
+        req = self._client.get("/datasets")
+        req.raise_for_status()
+        resp = req.json()
+        return [Dataset(id=dataset["id"], name=dataset["name"]) for dataset in resp]
+
+    def get_dataset(self, dataset_id: str) -> DatasetWithItems:
+        req = self._client.get(f"/datasets/{dataset_id}")
+        req.raise_for_status()
+        resp = req.json()
+        return DatasetWithItems(
+            id=resp["id"],
+            name=resp["name"],
+            items=[DatasetItem(id=item["id"], input=item["input"], output=item["output"]) for item in resp["items"]],
+        )
