@@ -27,6 +27,24 @@ class View:
     name: str
 
 
+@dataclass
+class Dataset:
+    id: str
+    name: str
+
+
+@dataclass
+class DatasetItem:
+    id: str
+    input: str
+    output: str
+
+
+@dataclass
+class DatasetWithItems(Dataset):
+    items: List[DatasetItem]
+
+
 @dataclass
 class TracesResponse:
     next_cursor: Optional[str]
@@ -62,6 +80,10 @@ class EventFilterOperator(StrEnum):
     NOT_CONTAINS = "NOT_CONTAINS"
     EQUALS = "EQUALS"
     NOT_EQUALS = "NOT_EQUALS"
+    LESS_THAN = "LESS_THAN"
+    LESS_THAN_OR_EQUALS = "LESS_THAN_OR_EQUALS"
+    GREATER_THAN = "GREATER_THAN"
+    GREATER_THAN_OR_EQUALS = "GREATER_THAN_OR_EQUALS"
 
 
 @dataclass
